#include "vmware.h"

/*
 * Détecte la présence de VMware. 
 * Retourne 1 si VMware est présent, 0 sinon
 */
int vmware_detect() {
  unsigned int vmware = 0;

  /* Ecriture de la valeur magique dans l'accumulateur */
  asm(" movl $0x564D5868, %eax; ");
  
  /* Ecriture de la commande désirée dans le registre CX */
  asm(" movw $0x000A, %cx;");

  /* Choix du port I/O de VMware dans le registre DX */
  asm(" movw $0x5658, %dx;");

  /*
   * Lecture du port I/O, c'est à cet instant que VMware 
   * répond à la demande
   */
  asm(" inl %dx, %eax;"); 

  /* 
   * Lecture de l'accumulateur contenant la taille de la
   * mémoire vive en Mo.
   * Il s'agit de la syntaxe de GNU as qui permet de lire
   * un registre directement dans une variable, ici vmware.
   */
  asm(" movl %%ebx, %0;" :"=r"(vmware));

  return (vmware == VMWARE_MAGIC);
}

/*
 * Retourne la fréquence du CPU en MHz selon VMware
 * Si VMware n'est pas détecté, 0xFFFFFFFF sera retourné
 */
int vmware_get_freq() {
  unsigned int freq = 0;
  
  /* Ecriture de la valeur magique dans l'accumulateur */
  asm(" movl $0x564D5868, %eax; ");

  /* Ecriture de la commande désirée dans le registre CX */
  asm(" movw $0x0001, %cx;");

  /* Choix du port I/O de VMware dans le registre DX */
  asm(" movw $0x5658, %dx;");

  /*
   * Lecture du port I/O, c'est à cet instant que VMware 
   * répond à la demande
   */
  asm(" inl %dx, %eax;"); 
  
  /* 
   * Lecture de l'accumulateur contenant la taille de la
   * mémoire vive en Mo.
   * Il s'agit de la syntaxe de GNU as qui permet de lire
   * un registre directement dans une variable, ici freq.
   */
  asm(" movl %%eax, %0;" :"=r"(freq));

  return freq;
}

/*
 * Retourne la taille de la RAM en Mo selon VMware
 * Si VMware n'est pas détecté, 0xFFFFFFFF sera retourné
 * Uniquement testé en mode 32 bits.
 */
int vmware_get_memsize() {
  unsigned int mem = 0;

  /* Ecriture de la valeur magique dans l'accumulateur */
  asm(" movl $0x564D5868, %eax; ");

  /* Ecriture de la commande désirée dans le registre CX */
  asm(" movw $0x0014, %cx;");

  /* Choix du port I/O de VMware dans le registre DX */
  asm(" movw $0x5658, %dx;");

  /*
   * Lecture du port I/O, c'est à cet instant que VMware 
   * répond à la demande
   */
  asm(" inl %dx, %eax;"); 

  /* 
   * Lecture de l'accumulateur contenant la taille de la
   * mémoire vive en Mo.
   * Il s'agit de la syntaxe de GNU as qui permet de lire
   * un registre directement dans une variable, ici mem.
   */
  asm(" movl %%eax, %0;" :"=r"(mem));

  return mem;
}
